package com.mortals.xhx.module.sst.web;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import cn.hutool.http.Method;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.ap.GlobalSysInfo;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.util.HttpUtil;
import com.mortals.framework.util.StringUtils;
import com.mortals.framework.web.BaseJsonBodyController;
import com.mortals.xhx.module.matter.service.UserMatterApplyService;
import com.mortals.xhx.module.sst.pdu.*;
import lombok.Getter;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.net.HttpCookie;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Base64;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.mortals.xhx.common.key.Constant.PARAM_SERVER_CWS_HTTP_URL;

/**
 * 综窗对接
 */
@RestController
public class SyntheticalController extends BaseJsonBodyController {

    @Value("${upload.path}")
    @Getter
    private String filePath;

    @Autowired
    private UserMatterApplyService userMatterApplyService;

    /**
     * 查询可受理事项
     * @param query
     * @return
     */
    @PostMapping({"api/event-implementation/list"})
    @UnAuth
    public String eventList(@RequestBody SyntheticalMatterQuery query) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/event-implementation/list";
        JSONObject jsonObject = new JSONObject();
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "查询可受理事项" ;
        try {
            resp = doPost(url, jsonObject.toJSONString());
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 查询事项详情
     * @param query
     * @return
     */
    @PostMapping({"api/event-implementation/get-by-id"})
    @UnAuth
    public String eventInfo(@RequestBody SyntheticalPdu query) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/event-implementation/get-by-id";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "查询事项详情" ;
        try {
            resp = doPost(url, JSONObject.toJSONString(query));
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 查询事项情形
     * @param query
     * @return
     */
    @PostMapping({"api/bus-situation-material/getSituationMaterialTreeByEventIds"})
    @UnAuth
    public String getSituationMaterialTreeByEventIds(@RequestBody SyntheticalPdu query) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/bus-situation-material/getSituationMaterialTreeByEventIds";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "查询事项情形" ;
        try {
            resp = doPost(url, JSONObject.toJSONString(query));
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 根据事项EventId查询办理事项材料
     * @param query
     * @return
     */
    @PostMapping({"api/material/findList"})
    @UnAuth
    public String findMaterialList(@RequestBody SyntheticalPdu query) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/material/findList";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "根据事项EventId查询办理事项材料" ;
        try {
            resp = doPost(url, JSONObject.toJSONString(query));
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 根据事项情形查询办理事项材料
     * @param query
     * @return
     */
    @PostMapping({"api/consulting-service/getMaterialByEventAndSituation"})
    @UnAuth
    public String getMaterialByEventAndSituation(@RequestBody SyntheticalPdu query) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/consulting-service/getMaterialByEventAndSituation";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "根据事项情形查询办理事项材料" ;
        try {
            resp = doPost(url, JSONObject.toJSONString(query));
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 获取办件要素
     * @param query
     * @return
     */
    @PostMapping({"api/event-implementation/getDynamicFormByEventIds"})
    @UnAuth
    public String getDynamicFormByEventIds(@RequestBody SyntheticalPdu query) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/event-implementation/getDynamicFormByEventIds";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "获取办件要素" ;
        try {
            resp = doPost(url, JSONObject.toJSONString(query));
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 查询字典
     * @param query
     * @return
     */
    @PostMapping({"api/common/queryDict"})
    @UnAuth
    public String queryDict(@RequestBody SyntheticalPdu query) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/common/queryDict";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "查询字典" ;
        try {
            resp = doPost(url, JSONObject.toJSONString(query));
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 自助设备接件
     * @param query
     * @return
     */
    @PostMapping({"api/self-device-info/acceptHandling"})
    @UnAuth
    public String acceptHandling(@RequestBody AcceptHandlingPdu query) {
        String url = GlobalSysInfo.getParamValue(PARAM_SERVER_CWS_HTTP_URL, "http://8.136.255.30:1086");
        url += "/complex/api/self-device-info/acceptHandling";
        String resp = null;
        Map<String, Object> model = new HashMap();
        String busiDesc = "自助设备接件" ;
        try {
            doFileToBase64(query);
            resp = doPost(url, JSONObject.toJSONString(query));
            this.recordSysLog(this.request, busiDesc + " 【成功】");
            JSONObject jsonObject = JSONObject.parseObject(resp);
            if(jsonObject.getIntValue("code")==1){

            }

        } catch (Exception e) {
            this.doException(this.request, busiDesc + " 【异常】", model, e);
            Rest<Object> ret = new Rest();
            ret.setCode(-1);
            ret.setMsg(busiDesc + " 【异常】");
            resp = JSON.toJSONString(ret);
        }
        return resp;
    }

    /**
     * 将上传文件地址转换为Base64
     * @param pdu
     */
    private void doFileToBase64(AcceptHandlingPdu pdu){
        if(ObjectUtil.isNotEmpty(pdu)){
            List<Material> materials = pdu.getMaterials();
            if(CollectionUtils.isNotEmpty(materials)){
                String rootPath = this.filePath.endsWith("/") ? this.filePath : this.filePath + "/";
                for(Material item:materials){
                    String filePath = rootPath + item.getMaterialFilePath();
                    try {
                        Path path = Paths.get(filePath);  // 替换为实际文件路径
                        byte[] fileBytes = Files.readAllBytes(path);
                        String base64EncodedString = Base64.getEncoder().encodeToString(fileBytes);
                        item.setMaterialContent(base64EncodedString);
                    } catch (Exception e) {
                        throw new AppException(e.getMessage());
                    }
                }
            }
        }
    }

    private String doPost(String url,String body) throws Exception{
        HttpRequest httpRequest = HttpRequest.post(url);
        httpRequest.body(body);
        HttpResponse execute = httpRequest.execute();

        // 6. 解析这个http响应类，可以获取到响应主体、cookie、是否请求成功等信息
//        boolean ok = execute.isOk(); // 是否请求成功 判断依据为：状态码范围在200~299内
//        System.out.println(ok);
//        List<HttpCookie> cookies = execute.getCookies();// 获取所有cookie
//        cookies.forEach(System.out::println); // 如果为空不会遍历的
        String resp = execute.body();   // 获取响应主体
        return resp;
    }

//    public static void main(String[] args){
//        String url = "http://8.136.255.30:1086/complex/api/event-implementation/get-by-id";
//        JSONObject jsonObject=new JSONObject();
//        System.out.println(jsonObject.toJSONString());
//        jsonObject.put("eventId","2e4575157ed27dbc7330b1595207b49c");
//        String resp = null;
//        Map<String, Object> model = new HashMap();
//        HashMap<String, String> paramsMap = new HashMap<>();
//        try {
//            HttpRequest httpRequest = HttpRequest.post(url);
//            httpRequest.body(jsonObject.toJSONString());
//            HttpResponse execute = httpRequest.execute();
//
//            // 6. 解析这个http响应类，可以获取到响应主体、cookie、是否请求成功等信息
//            boolean ok = execute.isOk(); // 是否请求成功 判断依据为：状态码范围在200~299内
//            System.out.println(ok);
//            List<HttpCookie> cookies = execute.getCookies();// 获取所有cookie
//            cookies.forEach(System.out::println); // 如果为空不会遍历的
//            String body = execute.body();   // 获取响应主体
//            System.out.println(body);
//
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//    }

//    public static void main(String[] args) {
//        try {
//            Path filePath = Paths.get("D:\\home\\mortals\\app\\aaa.jpg");  // 替换为实际文件路径
//            byte[] fileBytes = Files.readAllBytes(filePath);
//            String base64EncodedString = Base64.getEncoder().encodeToString(fileBytes);
//            System.out.println(base64EncodedString);
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//    }
}
